﻿/* /////////////////////////////////////////////////////////////////////////////////////////////////
MSAAToggle v0.2 by MacGruber.
Hotkeys to toggle VaM's MSAA settings, this helps with Reshade issues.

Version 0.2 2020-03-21
	Adding to MacGruber-Toolkit Package.
	Adding info text.
	
Version 0.1 2019-10-20
	Initial release.

///////////////////////////////////////////////////////////////////////////////////////////////// */


using UnityEngine;
using System.Collections;

namespace MacGruber
{
    public class MSAAToggle : MVRScript
    {
		private const int enabledMsaaLevel = 8;  // 2, 4 or 8
		private const float refreshWaitTime = 0.4f;
		
		public override void Init()
		{
			SetupInfoText(this, 
				"<color=#606060><size=40><b>MSAAToggle v0.2</b></size>\nHotkeys to toggle VaM's MSAA settings, this helps with Reshade issues.</color>\n\nPress L to toggle VaM's MSAA setting between OFF and 8x.\n\nPress K to toggle twice with a short delay in between.",
				500.0f, true
			);
		}

        private void Update()
		{
			if (Input.GetKeyDown(KeyCode.K)) // Refresh
				StartCoroutine(Refresh());
			else if (Input.GetKeyDown(KeyCode.L)) // Toggle
				Toggle();
        }
		
		private IEnumerator Refresh()
		{
			Toggle();
			yield return new WaitForSeconds(refreshWaitTime);
			Toggle();
		}
		
		private void Toggle()
		{
			bool isEnabled = UserPreferences.singleton.msaaLevel >= 2;
			UserPreferences.singleton.msaaLevel = isEnabled ? 0 : enabledMsaaLevel;
		}
		
		private static JSONStorableString SetupInfoText(MVRScript script, string text, float height, bool rightSide)
		{
			JSONStorableString storable = new JSONStorableString("Info", text);
			UIDynamic textfield = script.CreateTextField(storable, rightSide);
			textfield.height = height;
			return storable;
		}
    }
}
