﻿/* /////////////////////////////////////////////////////////////////////////////////////////////////
SkyMagic Exporter v0.1 by MacGruber.
Takes a cube-screenshot and saves it in SMC format. You will need to import this into Unity using SkyMagicImporter, then export back using an Unity AssetBundle, so you can load it again in VaM using SkyMagic Loader.

Version 0.1 2020-04-12
	Initial release.

///////////////////////////////////////////////////////////////////////////////////////////////// */

using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using UnityEngine.Rendering;
using System.Collections;
using mset;
using System;

namespace MacGruber
{
    public class SkyMagicExporter : MVRScript
    {	
		private JSONStorableFloat myResolution;
		private FreeControllerV3 myControl;
		
		
		public override void Init()
		{
			myControl = containingAtom.GetStorableByID("control") as FreeControllerV3;
			
			SetupInfoText(this,
				"<color=#606060><size=40><b>SkyMagic Exporter v0.1</b></size>\nTakes a cube-screenshot and saves it in SMC format. You will need to import this into Unity using SkyMagicImporter, then export back using an Unity AssetBundle, so you can load it again in VaM using SkyMagic Loader.</color>\n\n",
				600.0f, true
			);
			
			myResolution = SetupSliderFloat(this, "Resolution", 256, 64, 1024, false);
			myResolution.setCallbackFunction  += (float v) => { 
				myResolution.valNoCallback = Mathf.ClosestPowerOfTwo(Mathf.RoundToInt(v));
			};
				
			SetupButton(this, "Export Cubemap", FileBrowser, false);
		}
			
		private void FileBrowser()
		{	
			SuperController sc = SuperController.singleton;			
			sc.GetMediaPathDialog(ExportCubemap, "smc", "Saves/screenshots/", false, true, false, null, false, null, false, false);
			sc.mediaFileBrowserUI.SetTextEntry(true);
			if (sc.mediaFileBrowserUI.fileEntryField != null)
			{
				string filename = ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString();
				sc.mediaFileBrowserUI.fileEntryField.text = filename + ".smc";
				sc.mediaFileBrowserUI.ActivateFileNameField();
			}
		}
			
		private void ExportCubemap(string path)
		{
			if (!path.EndsWith(".smc"))
				path += ".smc";
			
			SuperController sc = SuperController.singleton;
			sc.ClearSelection(); // clear selection to avoid UI elements in the camera image.
			
			Camera screenshotCamera = sc.hiResScreenshotCamera;
			Camera cubemapCamera = myControl.GetComponent<Camera>();
			if (cubemapCamera == null)
				cubemapCamera = myControl.gameObject.AddComponent<Camera>();
			
			cubemapCamera.enabled = false;
			cubemapCamera.allowHDR = false;
			cubemapCamera.backgroundColor = screenshotCamera.backgroundColor;
			cubemapCamera.nearClipPlane = screenshotCamera.nearClipPlane;
			cubemapCamera.farClipPlane = screenshotCamera.farClipPlane;
			cubemapCamera.clearFlags = screenshotCamera.clearFlags;
			
			int uiMask = 1 << LayerMask.NameToLayer("UI")
			           | 1 << LayerMask.NameToLayer("LoadUI")
			           | 1 << LayerMask.NameToLayer("ScreenUI")
                       | 1 << LayerMask.NameToLayer("GUI");
			cubemapCamera.cullingMask = screenshotCamera.cullingMask & ~uiMask;	
			
			int s = Mathf.ClosestPowerOfTwo(Mathf.RoundToInt(myResolution.val));
			Cubemap cubemap = new Cubemap(s, TextureFormat.RGBA32, false);
			for (int i=(int)CubemapFace.PositiveX; i<=(int)CubemapFace.NegativeZ; ++i)
				cubemapCamera.RenderToCubemap(cubemap, 1 << i);			
							
			RenderTexture previousRT = RenderTexture.active;				
			
			Texture2D tex = new Texture2D(s*6, s, TextureFormat.RGB24, false);
			RenderTexture rtA = RenderTexture.GetTemporary(s, s, 0, RenderTextureFormat.ARGB32, RenderTextureReadWrite.sRGB, 1);
			RenderTexture rtB = RenderTexture.GetTemporary(s, s, 0, RenderTextureFormat.ARGB32, RenderTextureReadWrite.sRGB, 1);
			
			Rect sourceRect = new Rect(0, 0, s, s);
			for (int i=(int)CubemapFace.PositiveX; i<=(int)CubemapFace.NegativeZ; ++i)
			{
				RenderTexture.active = rtA;
				Graphics.CopyTexture(cubemap, i, 0, rtA, 0, 0);
				RenderTexture.active = rtB;
				Graphics.Blit(rtA, rtB, new Vector2(1, -1), new Vector2(0,1));
				tex.ReadPixels(sourceRect, i*s, 0, false);
			}
			tex.Apply();
			RenderTexture.active = previousRT;
			
			byte[] bytes = tex.EncodeToPNG();
			string data = Convert.ToBase64String(bytes);
			sc.SaveStringIntoFile(path, data);
							
			RenderTexture.ReleaseTemporary(rtA);
			RenderTexture.ReleaseTemporary(rtB);
			Destroy(tex);
			
			sc.SelectController(myControl);
		}
						
		private static void SetupButton(MVRScript script, string label, UnityAction callback, bool rightSide)
		{
			UIDynamicButton button = script.CreateButton(label, rightSide);
			button.button.onClick.AddListener(callback);
		}
		
		private static JSONStorableFloat SetupSliderFloat(MVRScript script, string label, float defaultValue, float minValue, float maxValue, bool rightSide)
		{
			JSONStorableFloat storable = new JSONStorableFloat(label, defaultValue, minValue, maxValue, true, true);
			storable.storeType = JSONStorableParam.StoreType.Full;
			script.CreateSlider(storable, rightSide);
			script.RegisterFloat(storable);
			return storable;
		}
		
		private static JSONStorableString SetupInfoText(MVRScript script, string text, float height, bool rightSide)
		{
			JSONStorableString storable = new JSONStorableString("Info", text);
			UIDynamic textfield = script.CreateTextField(storable, rightSide);
			textfield.height = height;
			return storable;
		}
    }
}
